@@ -543,6 +543,8 @@ class EasyEC2(EasyAWS):
         instance_type = instance.instance_type or 'N/A'
         keypair = instance.key_name or 'N/A'
         uptime = utils.get_elapsed_time(instance.launch_time) or 'N/A'
+        if state == 'stopped':
+            uptime = 'N/A'
         print "id: %s" % id
         print "dns_name: %s" % dns_name
         print "private_dns_name: %s" % private_dns_name
@@ -561,14 +563,15 @@ class EasyEC2(EasyAWS):
         print
 
     def list_all_instances(self, show_terminated=False):
+        tstates = ['shutting-down', 'terminated']
         insts = self.get_all_instances()
+        if not show_terminated:
+            insts = [i for i in insts if i.state not in tstates]
         if not insts:
             log.info("No instances found")
             return
-        tstates = ['shutting-down', 'terminated']
         for instance in insts:
-            if not instance.state in tstates or show_terminated:
-                self.show_instance(instance)
+            self.show_instance(instance)
 
     def list_images(self, images, sort_key=None, reverse=False):
         def get_key(obj):
