@@ -34,8 +34,6 @@ class CmdGet(ClusterCompleter):
                           help="Transfer files to NODE (defaults to master)")
 
     def execute(self, args):
-        if not self.cfg.globals.enable_experimental:
-            raise exception.ExperimentalFeature("The 'get' command")
         if len(args) < 3:
             self.parser.error("please specify a cluster, remote file or " +
                               "directory, and a local destination path")
