@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 from starcluster import static
 
 config_template = """\
@@ -34,6 +33,11 @@ AWS_USER_ID= #your userid
 # Uncomment these settings when creating an instance-store (S3) AMI (OPTIONAL)
 #EC2_CERT = /path/to/your/cert-asdf0as9df092039asdfi02089.pem
 #EC2_PRIVATE_KEY = /path/to/your/pk-asdfasd890f200909.pem
+# Uncomment these settings to use a proxy host when connecting to AWS
+#aws_proxy = your.proxyhost.com
+#aws_proxy_port = 8080
+#aws_proxy_user = yourproxyuser
+#aws_proxy_pass = yourproxypass
 
 # Sections starting with "key" define your keypairs
 # (see the EC2 getting started guide tutorial on using ec2-add-keypair to learn
@@ -62,9 +66,11 @@ CLUSTER_USER = sgeadmin
 # (options: %(shells)s)
 CLUSTER_SHELL = bash
 
-# AMI for cluster nodes.
+# AMI to use for cluster nodes. These AMIs are for the us-east-1 region.
+# Use the 'listpublic' command to list StarCluster AMIs in other regions
 # The base i386 StarCluster AMI is %(x86_ami)s
 # The base x86_64 StarCluster AMI is %(x86_64_ami)s
+# The base HVM StarCluster AMI is %(hvm_ami)s
 NODE_IMAGE_ID = %(x86_ami)s
 # instance type for all cluster nodes
 # (options: %(instance_types)s)
@@ -207,6 +213,7 @@ NODE_INSTANCE_TYPE = m1.small
 """ % {
     'x86_ami': static.BASE_AMI_32,
     'x86_64_ami': static.BASE_AMI_64,
+    'hvm_ami': static.BASE_AMI_HVM,
     'instance_types': ', '.join(static.INSTANCE_TYPES.keys()),
     'shells': ', '.join(static.AVAILABLE_SHELLS.keys()),
 }
