@@ -13,9 +13,9 @@ SPOOLING_METHOD="classic"
 DB_SPOOLING_SERVER="none"
 DB_SPOOLING_DIR="/opt/sge6/default/spooldb"
 PAR_EXECD_INST_COUNT="20"
-ADMIN_HOST_LIST="%s"
-SUBMIT_HOST_LIST="%s"
-EXEC_HOST_LIST="%s"
+ADMIN_HOST_LIST="%(admin_hosts)s"
+SUBMIT_HOST_LIST="%(submit_hosts)s"
+EXEC_HOST_LIST="%(exec_hosts)s"
 EXECD_SPOOL_DIR_LOCAL="/opt/sge6/default/spool/exec_spool_local"
 HOSTNAME_RESOLVING="true"
 SHELL_NAME="ssh"
